import { Entity } from '../entity'
import { ErrEntityNotFound, ErrFoundNotUniqEntity, Repo } from '../repo'
import { StorageSQLite } from '../storage/sqlite'

export abstract class RepoSQLite<T extends Entity<any>> extends Repo<T> {
    abstract Create(): Promise<void>

    async Drop(): Promise<void> {
        const st = this._storage as StorageSQLite
        await st?.db?.run(`DROP TABLE IF EXISTS ${this.Name()}`)
    }

    async Recreate(): Promise<void> {
        await this.Drop()
        await this.Create()
    }

    async findById(id: string): Promise<T> {
        const uniqKey = this.Entity().data.uniqKey()
        const cond = `${uniqKey} = ?`
        const cursor = await this._storage.find(this.Name(), { [cond]: [id] })
        const entryList = await cursor.toArray()
        if (entryList.length > 1) {
            throw new ErrFoundNotUniqEntity(
                `found few (${entryList.length}) entries in ${this.Name()} for id ${id}`
            )
        }

        if (entryList.length === 0) {
            throw new ErrEntityNotFound(`not found entry in [${this.Name()}] for id [${id}]`)
        }

        return this._transformer(entryList[0])
    }
}
