import { assert } from 'chai'
import { StorageSQLite } from '../../../app/storage/sqlite'

const itemsData = ['zero', 'one', 'two', 'three', 'four', 'five']
const dsn = __dirname + '/sqlsession.test.db'

describe('SQLiteSession', () => {
    let storage: StorageSQLite

    beforeAll(async () => {
        storage = new StorageSQLite({ dsn })
        await storage.connect()
        storage.db.on('trace', (data: string) => {
            console.log(data)
        })
        await storage.db.run('DROP TABLE IF EXISTS items')

        // UNIQUE нужно, чтобы работал UPSERT в методе save()
        await storage.db.run(
            'CREATE TABLE items (key varchar(8) NOT NULL UNIQUE, val integer NOT NULL DEFAULT 0)'
        )
        const stmt = await storage.db.prepare('INSERT INTO items (key, val) VALUES (?, ?)')
        for (let k = 0; k < itemsData.length; k++) {
            await stmt.run([k, itemsData[k]])
        }
    })

    describe('#commit', () => {
        it('should commit save', async () => {
            const session = storage.createSession()
            await session.start()
            await session.commit(async () => {
                await storage.save('items', 'key', { key: 'ten', val: 10 })
                await storage.save('items', 'key', { key: 'eleven', val: 11 })
                await storage.remove('items', 'key', 'ten')
            })

            const cursor = await storage.find('items', {})
            const items = await cursor.toArray()
            assert.equal(itemsData.length + 1, items.length)
            for (const item of items) {
                if (item['key'] === 'ten') assert.fail('ten had to be removed in session')
            }
        })
    })
})
